@@ -1,11 +1,11 @@
 import os
 
 import maitai._types as types
-from maitai._azure import MaiTaiAsyncAzureOpenAIClient as AsyncAzureOpenAI, MaitaiAzureOpenAIClient as AzureOpenAI
+from maitai._azure import MaitaiAsyncAzureOpenAIClient as AsyncAzureOpenAI, MaitaiAzureOpenAIClient as AzureOpenAI
 from maitai._evaluator import Evaluator as Evaluator
 from maitai._inference import Inference as Inference
-from maitai._openai import MaiTaiOpenAIClient as OpenAI
-from maitai._openai_async import MaiTaiAsyncOpenAIClient as AsyncOpenAI
+from maitai._openai import MaitaiOpenAIClient as OpenAI
+from maitai._openai_async import MaitaiAsyncOpenAIClient as AsyncOpenAI
 
 chat = OpenAI().chat
 
