@@ -3,7 +3,7 @@ import os
 import threading
 
 
-class MaiTaiClient:
+class MaitaiClient:
     MAITAI_HOST = os.environ.get("MAITAI_HOST", 'https://api.trymaitai.ai')
 
     def __init__(self):
