@@ -7,19 +7,19 @@ import openai.types.chat as openai_chat_types
 
 import maitai
 from maitai._config import config
+from maitai._evaluator import Evaluator
 from maitai._openai_types import Body, Headers, Query
 from maitai._types import EvaluateCallback
-from maitai._utils import convert_open_ai_chat_completion_chunk, convert_openai_chat_completion, get_chat_completion_params, process_chunk, process_stream_response, required_args, \
-    set_completion_evaluation_response
-from maitai_common.processes.stream_evaluation_thread import StreamEvaluationThread
+from maitai._utils import convert_open_ai_chat_completion_chunk, convert_openai_chat_completion, get_chat_completion_params, required_args
 from maitai_common.utils.proto_utils import openai_messages_to_proto
-from maitai_gen.chat import ChatCompletionChunk, ChatCompletionResponse, ChatMessage, EvaluateResponse, EvaluationContentType
+from maitai_gen.chat import ChatCompletionChunk, ChatCompletionResponse, ChatMessage, EvaluationContentType
+from maitai_gen.config import InferenceLocations
 from maitai_gen.inference import InferenceStreamResponse
 
 DEFAULT_MAX_RETRIES = 2
 
 
-class MaiTaiOpenAIClient:
+class MaitaiOpenAIClient:
     def __init__(
         self,
         *,
@@ -63,7 +63,8 @@ class Completions:
             client = openai.OpenAI()
         self.client = client
 
-    @required_args(["session_id", "action_type", "application_ref_name", "messages", "model"],
+    @required_args(["session_id", "action_type", "application_ref_name", "messages"],
+                   ["session_id", "action_type", "application_ref_name", "messages", "model"],
                    ["session_id", "action_type", "application_ref_name", "messages", "model", "stream"])
     def create(
         self,
@@ -74,10 +75,12 @@ class Completions:
         action_type: str = None,
         application_ref_name: str = None,
         callback: Optional[EvaluateCallback] = None,
-        server_side_inference: bool = False,  # TODO make this a config item
+        server_side_inference: bool = None,
+        evaluation_enabled: bool = None,
+        apply_corrections: bool = None,
         # OpenAI Arguments
         messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
-        model: Union[str, openai_types.ChatModel],
+        model: Union[str, openai_types.ChatModel, openai.NotGiven] = openai.NOT_GIVEN,
         frequency_penalty: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
         function_call: Union[openai_chat_types.completion_create_params.FunctionCall, openai.NotGiven] = openai.NOT_GIVEN,
         functions: Union[Iterable[openai_chat_types.completion_create_params.Function], openai.NotGiven] = openai.NOT_GIVEN,
@@ -89,7 +92,7 @@ class Completions:
         response_format: Union[openai_chat_types.completion_create_params.ResponseFormat, openai.NotGiven] = openai.NOT_GIVEN,
         seed: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
         stop: Union[Union[Optional[str], List[str]], openai.NotGiven] = openai.NOT_GIVEN,
-        stream: Optional[bool] = False,
+        stream: Union[Optional[bool], openai.NotGiven] = openai.NOT_GIVEN,
         stream_options: Union[Optional[openai_chat_types.ChatCompletionStreamOptionsParam], openai.NotGiven] = openai.NOT_GIVEN,
         temperature: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
         tool_choice: Union[openai_chat_types.ChatCompletionToolChoiceOptionParam, openai.NotGiven] = openai.NOT_GIVEN,
@@ -104,6 +107,33 @@ class Completions:
     ) -> Union[ChatCompletionResponse, Iterable[ChatCompletionChunk]]:
         if not config.api_key:
             raise ValueError("Maitai API Key has not been set")
+        if server_side_inference is False and apply_corrections is True:
+            raise ValueError("server_side_inference must be true to apply_corrections")
+        if apply_corrections is True and evaluation_enabled is False:
+            raise ValueError("evaluations must be enabled to apply_corrections")
+        maitai_config = config.get_application_action_config(application_ref_name, action_type)
+        if server_side_inference is None:
+            server_side_inference = maitai_config.inference_location == InferenceLocations.SERVER
+        if evaluation_enabled is None:
+            evaluation_enabled = maitai_config.evaluation_enabled
+        if apply_corrections is None:
+            apply_corrections = maitai_config.apply_corrections
+        if model is None:
+            model = maitai_config.model
+        if temperature == openai.NOT_GIVEN:
+            temperature = maitai_config.temperature
+        if stream is None:
+            stream = maitai_config.streaming
+        if response_format == openai.NOT_GIVEN:
+            response_format = {"type": maitai_config.response_format}
+        if stop == openai.NOT_GIVEN and maitai_config.stop is not None:
+            stop = maitai_config.stop
+        if logprobs == openai.NOT_GIVEN:
+            logprobs = maitai_config.logprobs
+        if max_tokens == openai.NOT_GIVEN and maitai_config.max_tokens is not None:
+            max_tokens = maitai_config.max_tokens
+        if n == openai.NOT_GIVEN:
+            n = maitai_config.n
         if server_side_inference:
             completion_params = get_chat_completion_params(
                 messages=messages,
@@ -129,20 +159,13 @@ class Completions:
             response_timeout = None
             if isinstance(timeout, float) or isinstance(timeout, int):
                 response_timeout = timeout
-            response = maitai.Inference.infer(session_id, reference_id, action_type, application_ref_name, completion_params, callback, timeout=response_timeout)
+            response = maitai.Inference.infer(session_id, reference_id, action_type, application_ref_name, completion_params, apply_corrections, evaluation_enabled, callback, timeout=response_timeout)
             if stream:
-                return _process_inference_stream(response, callback)
+                return _process_inference_stream(response)
             # ChatCompletion only
             chat_completion: Optional[ChatCompletionResponse] = None
             for resp in response:
-                if callback:
-                    return resp.chat_completion_response
-                else:
-                    if resp.chat_completion_response:
-                        chat_completion = resp.chat_completion_response
-                    if resp.evaluate_response and chat_completion:
-                        set_completion_evaluation_response(chat_completion, resp.evaluate_response)
-                        return chat_completion
+                return resp.chat_completion_response
             return chat_completion
         else:
             response = self.client.chat.completions.create(
@@ -174,23 +197,22 @@ class Completions:
 
             )
             if stream:
-                return _process_openai_stream(session_id, reference_id, action_type, application_ref_name, messages, response, callback)
+                return _process_openai_stream(session_id, reference_id, action_type, application_ref_name, messages, response, evaluation_enabled, callback)
             else:
                 maitai_completion = convert_openai_chat_completion(response)
                 proto_messages = openai_messages_to_proto(messages)
                 proto_messages.append(
                     ChatMessage(role="assistant", content=maitai_completion.choices[0].message.content))
-                maitai_eval = maitai.Evaluator.evaluate(
-                    session_id=session_id,
-                    reference_id=reference_id,
-                    action_type=action_type,
-                    content_type=EvaluationContentType.MESSAGE,
-                    content=proto_messages,
-                    application_ref_name=application_ref_name,
-                    callback=callback,
-                )
-                if not callback:
-                    set_completion_evaluation_response(maitai_completion, maitai_eval)
+                if evaluation_enabled:
+                    maitai.Evaluator.evaluate(
+                        session_id=session_id,
+                        reference_id=reference_id,
+                        action_type=action_type,
+                        content_type=EvaluationContentType.MESSAGE,
+                        content=proto_messages,
+                        application_ref_name=application_ref_name,
+                        callback=callback,
+                    )
                 return maitai_completion
 
 
@@ -200,41 +222,34 @@ def _process_openai_stream(session_id: Union[str, int],
                            application_ref_name: str,
                            messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
                            stream: openai.Stream[openai_chat_types.ChatCompletionChunk],
+                           evaluation_enabled: bool,
                            callback: Optional[EvaluateCallback] = None) -> Iterable[ChatCompletionChunk]:
     full_body = ""
     proto_messages = openai_messages_to_proto(messages)
-    stream_eval_thread = StreamEvaluationThread()
-    stream_eval_thread.start()
     for chunk in stream:
-        if stream_eval_thread.output:
-            result: EvaluateResponse = stream_eval_thread.output
-            stream_eval_thread.terminate()
-            new_stream = maitai.Evaluator.evaluate_realtime(session_id, reference_id, action_type, EvaluationContentType.PARTIAL, proto_messages, application_ref_name, full_body,
-                                                            result.evaluation_results[0].description, result.evaluation_results[0].sentinel_id)
-            for new_chunk in new_stream:
-                yield new_chunk
-            return
         maitai_chunk = convert_open_ai_chat_completion_chunk(chunk)
-        new_full_body, result_chunk = process_chunk(session_id, reference_id, action_type, application_ref_name, proto_messages, full_body, maitai_chunk, callback)
-        if new_full_body != full_body:
-            stream_eval_thread.input.append((session_id, reference_id, action_type, EvaluationContentType.PARTIAL, proto_messages, application_ref_name, new_full_body))
-        full_body = new_full_body
-        if result_chunk is not None:
-            yield result_chunk
-            break
+        content = maitai_chunk.choices[0].delta.content
+        if content is not None:
+            full_body += content
         yield maitai_chunk
-    stream_eval_thread.terminate()
+        if maitai_chunk.choices[0].finish_reason and evaluation_enabled:
+            proto_messages.append(ChatMessage(role="assistant", content=full_body))
+            Evaluator.evaluate(
+                session_id=session_id,
+                reference_id=reference_id,
+                action_type=action_type,
+                content_type=EvaluationContentType.MESSAGE,
+                content=proto_messages,
+                application_ref_name=application_ref_name,
+                callback=callback
+            )
+            return
 
 
-def _process_inference_stream(stream: Iterable[InferenceStreamResponse], callback: EvaluateCallback) -> Iterable[ChatCompletionChunk]:
-    last_chat_completion_chunk: Optional[ChatCompletionChunk] = None
+def _process_inference_stream(stream: Iterable[InferenceStreamResponse]) -> Iterable[ChatCompletionChunk]:
     for infer_resp in stream:
-        chunk = process_stream_response(infer_resp, callback, last_chat_completion_chunk)
+        chunk = infer_resp.chat_completion_chunk
         if chunk is not None:
-            if chunk.choices[0].finish_reason:
-                if callback:
-                    yield chunk
-                    return
-                else:
-                    last_chat_completion_chunk = chunk
             yield chunk
+            if chunk.choices[0].finish_reason:
+                return
